'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.actionWith = exports.normalizeTypeDescriptors = exports.getJSON = undefined;

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

/**
 * Extract JSON body from a server response
 *
 * @function getJSON
 * @access public
 * @param {object} res - A raw response object
 * @returns {promise|undefined}
 */
var getJSON = function () {
  var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(res) {
    var contentType, emptyCodes;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            contentType = res.headers.get('Content-Type');
            emptyCodes = [204, 205];

            if (!(!~emptyCodes.indexOf(res.status) && contentType && ~contentType.indexOf('json'))) {
              _context.next = 8;
              break;
            }

            _context.next = 5;
            return res.json();

          case 5:
            return _context.abrupt('return', _context.sent);

          case 8:
            _context.next = 10;
            return _promise2.default.resolve();

          case 10:
            return _context.abrupt('return', _context.sent);

          case 11:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function getJSON(_x) {
    return _ref.apply(this, arguments);
  };
}();

/**
 * Blow up string or symbol types into full-fledged type descriptors,
 *   and add defaults
 *
 * @function normalizeTypeDescriptors
 * @access private
 * @param {array} types - The [RSAA].types from a validated RSAA
 * @returns {array}
 */


/**
 * Evaluate a type descriptor to an FSA
 *
 * @function actionWith
 * @access private
 * @param {object} descriptor - A type descriptor
 * @param {array} args - The array of arguments for `payload` and `meta` function properties
 * @returns {object}
 */
var actionWith = function () {
  var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(descriptor, args) {
    var value, _value;

    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.prev = 0;

            if (!(typeof descriptor.payload === 'function')) {
              _context2.next = 8;
              break;
            }

            _context2.next = 4;
            return descriptor.payload.apply(descriptor, (0, _toConsumableArray3.default)(args));

          case 4:
            value = _context2.sent;

            descriptor.payload = function () {
              return value;
            };
            _context2.next = 11;
            break;

          case 8:
            _context2.next = 10;
            return descriptor.payload;

          case 10:
            descriptor.payload = _context2.sent;

          case 11:
            _context2.next = 17;
            break;

          case 13:
            _context2.prev = 13;
            _context2.t0 = _context2['catch'](0);

            if (typeof descriptor.payload === 'function') {
              descriptor.payload = function () {
                return new _errors.InternalError(_context2.t0.message);
              };
            } else {
              descriptor.payload = new _errors.InternalError(_context2.t0.message);
            }
            descriptor.error = true;

          case 17:
            _context2.prev = 17;

            if (!(typeof descriptor.meta === 'function')) {
              _context2.next = 25;
              break;
            }

            _context2.next = 21;
            return descriptor.meta.apply(descriptor, (0, _toConsumableArray3.default)(args));

          case 21:
            _value = _context2.sent;

            descriptor.meta = function () {
              return _value;
            };
            _context2.next = 28;
            break;

          case 25:
            _context2.next = 27;
            return descriptor.meta;

          case 27:
            descriptor.meta = _context2.sent;

          case 28:
            _context2.next = 35;
            break;

          case 30:
            _context2.prev = 30;
            _context2.t1 = _context2['catch'](17);

            delete descriptor.meta;
            if (typeof descriptor.payload === 'function') {
              descriptor.payload = function () {
                return new _errors.InternalError(_context2.t1.message);
              };
            } else {
              descriptor.payload = new _errors.InternalError(_context2.t1.message);
            }
            descriptor.error = true;

          case 35:
            return _context2.abrupt('return', descriptor);

          case 36:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this, [[0, 13], [17, 30]]);
  }));

  return function actionWith(_x2, _x3) {
    return _ref2.apply(this, arguments);
  };
}();

var _errors = require('./errors');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function normalizeTypeDescriptors(types) {
  var _types = (0, _slicedToArray3.default)(types, 3),
      requestType = _types[0],
      successType = _types[1],
      failureType = _types[2];

  if (typeof requestType === 'string' || (typeof requestType === 'undefined' ? 'undefined' : (0, _typeof3.default)(requestType)) === 'symbol') {
    requestType = { type: requestType };
  }

  if (typeof successType === 'string' || (typeof successType === 'undefined' ? 'undefined' : (0, _typeof3.default)(successType)) === 'symbol') {
    successType = { type: successType };
  }

  successType.payload = successType.payload || function (action, state, res) {
    return getJSON(res);
  };

  if (typeof failureType === 'string' || (typeof failureType === 'undefined' ? 'undefined' : (0, _typeof3.default)(failureType)) === 'symbol') {
    failureType = { type: failureType };
  }

  failureType.payload = failureType.payload || function (action, state, res) {
    return getJSON(res).then(function (json) {
      return new _errors.ApiError(res.status, res.statusText, json);
    });
  };

  return [requestType, successType, failureType];
}exports.getJSON = getJSON;
exports.normalizeTypeDescriptors = normalizeTypeDescriptors;
exports.actionWith = actionWith;